@@ -33,12 +33,15 @@ extern "C"
 #include "rmw/ret_types.h"
 #include "rmw/security_options.h"
 #include "rmw/serialized_message.h"
+#include "rmw/subscription_content_filter_options.h"
 #include "rmw/time.h"
 #include "rmw/visibility_control.h"
 
-// 24 bytes is the most memory needed to represent the GID by any current
-// implementation. It may need to be increased in the future.
-#define RMW_GID_STORAGE_SIZE 24u
+// We define the GID as 16 bytes (128 bits).  This should be enough to ensure
+// uniqueness amongst all entities in the system.  It is up to the individual
+// RMW implementations to fill that in, either from the underlying middleware
+// or from the RMW layer itself.
+#define RMW_GID_STORAGE_SIZE 16u
 
 /// Structure which encapsulates an rmw node
 typedef struct RMW_PUBLIC_TYPE rmw_node_s
@@ -177,6 +180,9 @@ typedef struct RMW_PUBLIC_TYPE rmw_subscription_options_s
    * Default value is RMW_UNIQUE_NETWORK_FLOW_ENDPOINTS_NOT_REQUIRED.
    */
   rmw_unique_network_flow_endpoints_requirement_t require_unique_network_flow_endpoints;
+
+  /// Used to create a content filter options during subscription creation.
+  rmw_subscription_content_filter_options_t * content_filter_options;
 } rmw_subscription_options_t;
 
 typedef struct RMW_PUBLIC_TYPE rmw_subscription_s
@@ -203,6 +209,9 @@ typedef struct RMW_PUBLIC_TYPE rmw_subscription_s
 
   /// Indicates whether this subscription can loan messages
   bool can_loan_messages;
+
+  /// Indicates whether content filtered topic of this subscription is enabled
+  bool is_cft_enabled;
 } rmw_subscription_t;
 
 /// A handle to an rmw service
@@ -349,7 +358,7 @@ typedef struct RMW_PUBLIC_TYPE rmw_wait_set_s
 typedef struct RMW_PUBLIC_TYPE rmw_request_id_s
 {
   /// The guid of the writer associated with this request
-  int8_t writer_guid[16];
+  uint8_t writer_guid[RMW_GID_STORAGE_SIZE];
 
   /// Sequence number of this service
   int64_t sequence_number;
@@ -375,7 +384,26 @@ typedef enum RMW_PUBLIC_TYPE rmw_qos_reliability_policy_e
   RMW_QOS_POLICY_RELIABILITY_BEST_EFFORT,
 
   /// Reliability policy has not yet been set
-  RMW_QOS_POLICY_RELIABILITY_UNKNOWN
+  RMW_QOS_POLICY_RELIABILITY_UNKNOWN,
+
+  /// Will match the majority of endpoints and use a reliable policy if possible
+  /**
+   * A policy will be chosen at the time of creating a subscription or publisher.
+   * A reliable policy will by chosen if it matches with all discovered endpoints,
+   * otherwise a best effort policy will be chosen.
+   *
+   * The QoS policy reported by functions like `rmw_subscription_get_actual_qos` or
+   * `rmw_publisher_get_actual_qos` may be best available, reliable, or best effort.
+   *
+   * Services and clients are not supported and default to the reliability value in
+   * `rmw_qos_profile_services_default`.
+   *
+   * The middleware is not expected to update the policy after creating a subscription or
+   * publisher, even if the chosen policy is incompatible with newly discovered endpoints.
+   * Therefore, this policy should be used with care since non-deterministic behavior
+   * can occur due to races with discovery.
+   */
+  RMW_QOS_POLICY_RELIABILITY_BEST_AVAILABLE
 } rmw_qos_reliability_policy_t;
 
 /// QoS history enumerations describing how samples endure
@@ -407,7 +435,29 @@ typedef enum RMW_PUBLIC_TYPE rmw_qos_durability_policy_e
   RMW_QOS_POLICY_DURABILITY_VOLATILE,
 
   /// Durability policy has not yet been set
-  RMW_QOS_POLICY_DURABILITY_UNKNOWN
+  RMW_QOS_POLICY_DURABILITY_UNKNOWN,
+
+  /// Will match the majority of endpoints and use a transient local policy if possible
+  /**
+   * A policy will be chosen at the time of creating a subscription or publisher.
+   * A transient local policy will by chosen if it matches with all discovered endpoints,
+   * otherwise a volatile policy will be chosen.
+   *
+   * In the case that a volatile policy is chosen for a subscription, any messages sent before
+   * the subscription was created by transient local publishers will not be received.
+   *
+   * The QoS policy reported by functions like `rmw_subscription_get_actual_qos` or
+   * `rmw_publisher_get_actual_qos` may be best available, transient local, or volatile.
+   *
+   * Services and clients are not supported and default to the durability value in
+   * `rmw_qos_profile_services_default`.
+   *
+   * The middleware is not expected to update the policy after creating a subscription or
+   * publisher, even if the chosen policy is incompatible with newly discovered endpoints.
+   * Therefore, this policy should be used with care since non-deterministic behavior
+   * can occur due to races with discovery.
+   */
+  RMW_QOS_POLICY_DURABILITY_BEST_AVAILABLE
 } rmw_qos_durability_policy_t;
 
 #define RMW_QOS_POLICY_LIVELINESS_MANUAL_BY_NODE_DEPRECATED_MSG \
@@ -446,17 +496,80 @@ typedef enum RMW_PUBLIC_TYPE rmw_qos_liveliness_policy_e
   RMW_QOS_POLICY_LIVELINESS_MANUAL_BY_TOPIC = 3,
 
   /// Liveliness policy has not yet been set
-  RMW_QOS_POLICY_LIVELINESS_UNKNOWN = 4
+  RMW_QOS_POLICY_LIVELINESS_UNKNOWN = 4,
+
+  /// Will match the majority of endpoints and use a manual by topic policy if possible
+  /**
+   * A policy will be chosen at the time of creating a subscription or publisher.
+   * A manual by topic policy will by chosen if it matches with all discovered endpoints,
+   * otherwise an automatic policy will be chosen.
+   *
+   * The QoS policy reported by functions like `rmw_subscription_get_actual_qos` or
+   * `rmw_publisher_get_actual_qos` may be best available, automatic, or manual by topic.
+   *
+   * Services and clients are not supported and default to the liveliness value in
+   * `rmw_qos_profile_services_default`.
+   *
+   * The middleware is not expected to update the policy after creating a subscription or
+   * publisher, even if the chosen policy is incompatible with newly discovered endpoints.
+   * Therefore, this policy should be used with care since non-deterministic behavior
+   * can occur due to races with discovery.
+   */
+  RMW_QOS_POLICY_LIVELINESS_BEST_AVAILABLE = 5
 } rmw_qos_liveliness_policy_t;
 
 /// QoS Deadline default.
 #define RMW_QOS_DEADLINE_DEFAULT RMW_DURATION_UNSPECIFIED
+/// Will match the majority of endpoints while maintaining as strict a policy as possible
+/**
+ * Value is RMW_DURATION_INFINITE - 1.
+ *
+ * A policy will be chosen at the time of creating a subscription or publisher.
+ * For a subscription, the deadline will be the maximum value of all discovered publisher
+ * deadlines.
+ * For a publisher, the deadline will be the minimum value of all discovered subscription
+ * deadlines.
+ *
+ * The QoS policy reported by functions like `rmw_subscription_get_actual_qos` or
+ * `rmw_publisher_get_actual_qos` may be best available or the actual deadline value.
+ *
+ * Services and clients are not supported and default to the deadline value in
+ * `rmw_qos_profile_services_default`.
+ *
+ * The middleware is not expected to update the policy after creating a subscription or
+ * publisher, even if the chosen policy is incompatible with newly discovered endpoints.
+ * Therefore, this policy should be used with care since non-deterministic behavior
+ * can occur due to races with discovery.
+ */
+#define RMW_QOS_DEADLINE_BEST_AVAILABLE {9223372036LL, 854775806LL}
 
 /// QoS Lifespan default.
 #define RMW_QOS_LIFESPAN_DEFAULT RMW_DURATION_UNSPECIFIED
 
 /// QoS Liveliness lease duration default.
 #define RMW_QOS_LIVELINESS_LEASE_DURATION_DEFAULT RMW_DURATION_UNSPECIFIED
+/// Will match the majority of endpoints while maintaining as strict a policy as possible
+/**
+ * Value is RMW_DURATION_INFINITE - 1.
+ *
+ * A policy will be chosen at the time of creating a subscription or publisher.
+ * For a subscription, the lease duration will be the maximum value of all discovered publisher
+ * lease durations.
+ * For a publisher, the lease duration will be the minimum value of all discovered subscription
+ * lease durations.
+ *
+ * The QoS policy reported by functions like `rmw_subscription_get_actual_qos` or
+ * `rmw_publisher_get_actual_qos` may be best available or the actual lease duration value.
+ *
+ * Services and clients are not supported and default to the lease duration value in
+ * `rmw_qos_profile_services_default`.
+ *
+ * The middleware is not expected to update the policy after creating a subscription or
+ * publisher, even if the chosen policy is incompatible with newly discovered endpoints.
+ * Therefore, this policy should be used with care since non-deterministic behavior
+ * can occur due to races with discovery.
+ */
+#define RMW_QOS_LIVELINESS_LEASE_DURATION_BEST_AVAILABLE {9223372036LL, 854775806LL}
 
 /// ROS MiddleWare quality of service profile.
 typedef struct RMW_PUBLIC_TYPE rmw_qos_profile_s
@@ -505,21 +618,104 @@ typedef struct RMW_PUBLIC_TYPE rmw_qos_profile_s
   bool avoid_ros_namespace_conventions;
 } rmw_qos_profile_t;
 
-/// ROS graph ID of the topic
+/// Globally unique identifier for a ROS graph entity
+/**
+ * This is expected to be globally unique within a ROS domain.
+ * The identifier should be the same when reported both locally (where the entity was created)
+ * and on remote hosts or processes.
+ */
 typedef struct RMW_PUBLIC_TYPE rmw_gid_s
 {
   /// Name of the rmw implementation
   const char * implementation_identifier;
 
-  /// Bype data Gid value
+  /// Byte data GID value
   uint8_t data[RMW_GID_STORAGE_SIZE];
 } rmw_gid_t;
 
+#define RMW_MESSAGE_INFO_SEQUENCE_NUMBER_UNSUPPORTED UINT64_MAX
+
 /// Information describing an rmw message
 typedef struct RMW_PUBLIC_TYPE rmw_message_info_s
 {
+  /// Time when the message was published by the publisher.
+  /**
+   * The exact point at which the timestamp is taken is not specified, but
+   * it should be taken consistently at the same point in the
+   * publishing process each time.
+   */
   rmw_time_point_value_t source_timestamp;
+  /// Time when the message was received by the subscription.
+  /**
+   * The exact point at which the timestamp is taken is not specified, but
+   * it should be taken consistently at the same point in the
+   * process of receiving a message each time.
+   */
   rmw_time_point_value_t received_timestamp;
+  /// Sequence number of the received message set by the publisher.
+  /**
+   * This sequence number is set by the publisher and therefore uniquely identifies
+   * a message when combined with the publisher GID.
+   * For long running applications, the sequence number might wrap around at some point.
+   *
+   * If the rmw implementation doesn't support sequence numbers, its value will be
+   * RMW_MESSAGE_INFO_SEQUENCE_NUMBER_UNSUPPORTED.
+   *
+   * Requirements:
+   *
+   * If `psn1` and `psn2` are the publication sequence numbers obtained by
+   * calls to `rmw_take*()`, where `psn1` was obtained in a call that happened before `psn2` and both
+   * sequence numbers are from the same publisher (i.e. also same publisher gid), then:
+   *
+   * - psn2 > psn1 (except in the case of a wrap around)
+   * - `psn2 - psn1 - 1` is the number of messages the publisher sent in the middle of both
+   *   received messages.
+   *   Those might have already been taken by other `rmw_take*()` calls that happened in between or lost.
+   *   `psn2 - psn1 - 1 = 0` if and only if the messages were sent by the publisher consecutively.
+   */
+  uint64_t publication_sequence_number;
+  /// Sequence number of the received message set by the subscription.
+  /**
+   * This sequence number is set by the subscription regardless of which
+   * publisher sent the message.
+   * For long running applications, the sequence number might wrap around at some point.
+   *
+   * If the rmw implementation doesn't support sequence numbers, its value will be
+   * RMW_MESSAGE_INFO_SEQUENCE_NUMBER_UNSUPPORTED.
+   *
+   * Requirements:
+   *
+   * If `rsn1` and `rsn2` are the reception sequence numbers obtained by
+   * calls to `rmw_take*()`, where `rsn1` was obtained in a call that happened before `rsn2`, then:
+   *
+   * - rsn2 > rsn1 (except in the case of a wrap around)
+   * - `rsn2 = rsn1 + 1` if and only if both `rmw_take*()` calls happened consecutively.
+   */
+  uint64_t reception_sequence_number;
+  /// Global unique identifier of the publisher that sent the message.
+  /**
+   * The identifier uniquely identifies the publisher for the local context, but
+   * it will not necessarily be the same identifier given in other contexts or processes
+   * for the same publisher.
+   * Therefore the identifier will uniquely identify the publisher within your application
+   * but may disagree about the identifier for that publisher when compared to another
+   * application.
+   * Even with this limitation, when combined with the publisher sequence number it can
+   * uniquely identify a message within your local context.
+   * Publisher GIDs generated by the rmw implementation could collide at some point, in which
+   * case it is not possible to distinguish which publisher sent the message.
+   * The details of how GIDs are generated are rmw implementation dependent.
+   *
+   * It is possible the the rmw implementation needs to reuse a publisher GID,
+   * due to running out of unique identifiers or some other constraint, in which case
+   * the rmw implementation may document what happens in that case, but that
+   * behavior is not defined here.
+   * However, this should be avoided, if at all possible, by the rmw implementation,
+   * and should be unlikely to happen in practice.
+   *
+   * \todo In the future we want this to uniquely identify the publisher globally across
+   *   contexts, processes, and machines.
+   */
   rmw_gid_t publisher_gid;
 
   /// Whether this message is from intra_process communication or not
